/*
 * *****************************************************************************************
 *
 * 		Copyright (C) 2014-2021 Gowin Semiconductor Technology Co.,Ltd.
 * 		
 * @file      main.c
 * @author    Embedded Development Team
 * @version   V1.x.x
 * @date      2021-01-01 09:00:00
 * @brief     Main function.
 ******************************************************************************************
 */
/* Includes ------------------------------------------------------------------*/
#include "gw1ns4c.h"

/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include "hyper_ram.h"

/* Includes ------------------------------------------------------------------*/
void UartInit(void);
int memtest(uint32_t startaddress, uint32_t stoppaddress);
int delmem(uint32_t startaddress, uint32_t stoppaddress);
int xatoi (			/* 0:Failed, 1:Successful */
	char **str,		/* Pointer to pointer to the string */
	long *res		/* Pointer to the variable to store the value */
);
/* Macros ------------------------------------------------------------------*/


/* Variables ------------------------------------------------------------------*/
uint32_t rec_temp[4];
uint32_t temp[4] = { 0x00};

// RX Buffer
char Terminal;
unsigned char RXBuf[64];
unsigned char CmdBuf[64];
signed char RXpt;
signed char RXptLastCommand;
char RXReceivedFlag;

/* Functions ------------------------------------------------------------------*/
int main()
{
	NVIC_InitTypeDef InitTypeDef_NVIC;

	//Enable UART0 interrupt handler
	InitTypeDef_NVIC.NVIC_IRQChannel = UART0_IRQn;
	InitTypeDef_NVIC.NVIC_IRQChannelPreemptionPriority = 1;
	InitTypeDef_NVIC.NVIC_IRQChannelSubPriority = 1;
	InitTypeDef_NVIC.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&InitTypeDef_NVIC);

	SystemInit();	//Initializes system
	UartInit();		//Initializes UART0
	Terminal = SET;
/*
	printf("\r\n---------------------------------------------------------------------\r\n");
	printf("       GowinSemiconductor Gowin_EMPU(GW1NS-4C) HyperRAM Application      \r\n");
	printf("---------------------------------------------------------------------\r\n\r\n");

	printf("System Initialized Successfully!\r\n");
	printf("UART0 Initialized Successfully!\r\n");
*/
	//Check and wait HyperRAM initialization finished
	while(HPRAM_Check_Init_Status()!=1);
	printf("HyperRAM Initialized!\r\n");


	while(1)
	{
		int i, ii;
		int iParam;

		char *ptr;
		unsigned long address, startaddress, stoppaddress;
		unsigned long size;
		int result;
		uint32_t temp[4]={0x00};
		unsigned long erraddress;
		uint32_t rxtemp[4]={0x00};
		char error=0;


		if(RXReceivedFlag==1)
		{
			RXReceivedFlag = 0;
			UART_SendChar(UART0, RXBuf[RXpt]);
			if(RXBuf[RXpt]=='\r') //command received
			{
				for(i=0; i<64; i++)
				{
					RXptLastCommand++;
					if (RXptLastCommand > 63)
						RXptLastCommand = 0;

					if(RXBuf[RXptLastCommand] == '\r') // end of command
					{
						CmdBuf[i] = 0;
						break;
					}
					else
						CmdBuf[i] = RXBuf[RXptLastCommand];
				}

				// Command in CmdBuf

				printf("\r\n");

				if (CmdBuf[0]==0)
					asm("nop");

				else if(!strncmp(CmdBuf, "memtest",7))
				{
					ptr = &CmdBuf[7];

					result = xatoi(&ptr, &startaddress);
					result += xatoi(&ptr, &stoppaddress);

					if (result == 2)
					{
						printf("memtest %u %u\r\n", startaddress, stoppaddress);
						memtest(startaddress, stoppaddress);
					}
					else
						printf("eg. memtest 10 2000 - 10..Startaddress, 2000...Endaddress\r\n");
				}

				else if(!strncmp(CmdBuf,"r ",2))
				{

					ptr = &CmdBuf[2];

					result = xatoi(&ptr, &address);

					if (result == 1)
					{
						printf("r %u \r\n", address);

						//Read Data From HyperRAM
						HPRAM_Read_Data_Buff(rec_temp,address);
						printf("0x%08x:  0x %08x %08x %08x %08x\r\n",address, rec_temp[0], rec_temp[1], rec_temp[2], rec_temp[3]);

					}
					else
						printf("eg. r 0x2000\r\n");
				}

				else if(!strncmp(CmdBuf,"d",1))
				{

					ptr = &CmdBuf[2];

					result = xatoi(&ptr, &startaddress);
					result += xatoi(&ptr, &stoppaddress);

					if (result == 2)
					{
						printf("d %u %u\r\n", startaddress, stoppaddress);
						delmem(startaddress, stoppaddress);
					}
					else
						printf("eg. d 10 200 - 10..Start, 200...End\r\n");
				}

				else if(!strncmp(CmdBuf,"w",1))
				{

					ptr = &CmdBuf[2];

					result = xatoi(&ptr, &address);
					result += xatoi(&ptr, &temp[0]);
					result += xatoi(&ptr, &temp[1]);
					result += xatoi(&ptr, &temp[2]);
					result += xatoi(&ptr, &temp[3]);

					if (result == 5)
					{
						printf("d %08x : %08x %08x %08x %08x\r\n", address, temp[0], temp[1], temp[2], temp[3]);
						HPRAM_Write_Data_Buff(temp,address);
					}
					else
						printf("eg. w 10 0x1... \r\n");
				}

				else if(!strncmp(CmdBuf,"gbreset 1",9))
				{

					printf("reset on\r\n");
					HPRAM_GB_RESET_Set(0);
				}
				else if(!strncmp(CmdBuf,"gbreset 0",9))
				{

					printf("reset off\r\n");
					HPRAM_GB_RESET_Set(1);
				}

				else if(!strncmp(CmdBuf,"gbr",3))
				{
					HPRAM_GB_RESET_Set(0);
					for (i= 0; i< 30000; i++);
					HPRAM_GB_RESET_Set(1);
					printf("reset off\r\n");
				}

				else if(!strncmp(CmdBuf,"gbmode 1",8))
				{

					printf("GB Mode on\r\n");
					HPRAM_GB_nAHB_MODE_Set(1);
				}
				else if(!strncmp(CmdBuf,"gbmode 0",8))
				{

					printf("GB Mode off\r\n");
					HPRAM_GB_nAHB_MODE_Set(0);
				}

				else if(!strncmp(CmdBuf,"sendfile",8))
				{


					i=0;

					ptr = &CmdBuf[8];

					result = xatoi(&ptr, &address);
					result += xatoi(&ptr, &size);

					if (result == 2)
					{
						printf("receiving %lu bytes\r\n", size);

						Terminal = RESET;

						while(size)
						{
							if(RXReceivedFlag)
							{
								RXReceivedFlag = 0;

								switch (i)
								{
									case 0: temp[0] = RXBuf[RXpt] << 16; break;
									case 1: temp[0] += RXBuf[RXpt]; break;
									case 2: temp[1] = RXBuf[RXpt] << 16; break;
									case 3: temp[1] += RXBuf[RXpt]; break;
									case 4: temp[2] = RXBuf[RXpt] << 16; break;
									case 5: temp[2] += RXBuf[RXpt]; break;
									case 6: temp[3] = RXBuf[RXpt] << 16; break;
									case 7: temp[3] += RXBuf[RXpt]; break;
								}

								i++;

								if (i==8)
								{
									i = 0;
									HPRAM_Write_Data_Buff(temp, address);
									if(size > 7)
										size -= 8;
									else
										size = 0;

									address += 8;
								}
							}
						}

						Terminal = SET;

						printf("complete\r\n", address, size);
					}
					else
						printf("eg. sendfile 0x0 64 (multiple of 8) \r\n");
				}

/*
				else if(!strncmp(CmdBuf,"verify",6))
				{

					i=0;
					error = 0;

					ptr = &CmdBuf[6];

					result = xatoi(&ptr, &address);
					result += xatoi(&ptr, &size);

					if (result == 2)
					{
						printf("receiving %u bytes\r\n", size);

						Terminal = RESET;

						while(size)
						{
							if(RXReceivedFlag)
							{
								RXReceivedFlag = 0;

								switch (i)
								{
									case 0: temp[0] = RXBuf[RXpt] << 16; break;
									case 1: temp[0] += RXBuf[RXpt]; break;
									case 2: temp[1] = RXBuf[RXpt] << 16; break;
									case 3: temp[1] += RXBuf[RXpt]; break;
									case 4: temp[2] = RXBuf[RXpt] << 16; break;
									case 5: temp[2] += RXBuf[RXpt]; break;
									case 6: temp[3] = RXBuf[RXpt] << 16; break;
									case 7: temp[3] += RXBuf[RXpt]; break;
								}

								i++;

								if (i==8)
								{
									i = 0;
									HPRAM_Read_Data_Buff(rxtemp, address);

									if(rxtemp[0] != temp[0] || rxtemp[1] != temp[1] || rxtemp[2] != temp[2] || rxtemp[3] != temp[3])
									{
										printf("err %08x %08x %08x %08x != %08x %08x %08x %08x\r\n", temp[0], temp[1], temp[2], temp[3], rxtemp[0], rxtemp[1], rxtemp[2], rxtemp[3]);
										error = 1;
										if (!error)
											erraddress = address;
									}

									if(size > 7)
										size -= 8;
									else
										size = 0;

									address += 8;
								}
							}
						}

						Terminal = SET;

						if(error)
							printf("err %u\r\r", erraddress);
						else
							printf("complete\r\n", address, size);
					}
					else
						printf("verify\r\n");
				}
*/

				else
				{
					printf("Command '%s' is unknown\r\n", CmdBuf);
				}
			}
		}
		else if (RXReceivedFlag == 2) // Backspace
		{
			RXReceivedFlag = 0;
			UART_SendChar(UART0, (char)8);
		}
	}
}

/*----------------------------------------------*/
/* Get a value of the string                    */
/*----------------------------------------------*/
/*	"123 -5   0x3ff 0b1111 0377  w "
	    ^                           1st call returns 123 and next ptr
	       ^                        2nd call returns -5 and next ptr
                   ^                3rd call returns 1023 and next ptr
                          ^         4th call returns 15 and next ptr
                               ^    5th call returns 255 and next ptr
                                  ^ 6th call fails and returns 0
*/


int xatoi (			/* 0:Failed, 1:Successful */
	char **str,		/* Pointer to pointer to the string */
	long *res		/* Pointer to the variable to store the value */
)
{
	unsigned long val;
	unsigned char c, r, s = 0;


	*res = 0;

	while ((c = **str) == ' ') (*str)++;	/* Skip leading spaces */

	if (c == '-') {		/* negative? */
		s = 1;
		c = *(++(*str));
	}

	if (c == '0') {
		c = *(++(*str));
		switch (c) {
		case 'x':		/* hexdecimal */
			r = 16; c = *(++(*str));
			break;
		case 'b':		/* binary */
			r = 2; c = *(++(*str));
			break;
		default:
			if (c <= ' ') return 1;	/* single zero */
			if (c < '0' || c > '9') return 0;	/* invalid char */
			r = 8;		/* octal */
		}
	} else {
		if (c < '0' || c > '9') return 0;	/* EOL or invalid char */
		r = 10;			/* decimal */
	}

	val = 0;
	while (c > ' ') {
		if (c >= 'a') c -= 0x20;
		c -= '0';
		if (c >= 17) {
			c -= 7;
			if (c <= 9) return 0;	/* invalid char */
		}
		if (c >= r) return 0;		/* invalid char for current radix */
		val = val * r + c;
		c = *(++(*str));
	}
	if (s) val = 0 - val;			/* apply sign if needed */

	*res = val;
	return 1;
}

int memtest(uint32_t startaddress, uint32_t stoppaddress)
{
	uint32_t rec_temp[4];
	uint32_t temp[4] = { 0x00};

	//Write data into HyperRAM
	printf("\r\nWrite\r\n");
	for(uint32_t i = startaddress; i < stoppaddress; i = i+8)
	{
		for(uint16_t j = 0; j < 4; j++)
		{
			temp[j] = (i + j*2 + 1) + ((i + j*2)<<16) ;
		}
		printf("0x%08x : 0x %08x %08x %08x %08x\r\n",i, temp[0], temp[1], temp[2], temp[3]);

		HPRAM_Write_Data_Buff(temp,i);
	}

	//Read Data From HyperRAM
	printf("\r\nRead\r\n");
	for(uint32_t i = startaddress; i < stoppaddress; i = i+8)
	{
		HPRAM_Read_Data_Buff(rec_temp,i);
		printf("0x%08x : 0x %08x %08x %08x %08x\r\n",i, rec_temp[0], rec_temp[1], rec_temp[2], rec_temp[3]);

	}

	printf("\r\nFinished\r\n");

	return 0;
}

int delmem(uint32_t startaddress, uint32_t stoppaddress)
{
	uint32_t rec_temp[4];
	uint32_t temp[4] = { 0x00};

	//Write data into HyperRAM
	for(uint32_t i = startaddress; i < stoppaddress; i = i+8)
	{
		for(uint16_t j = 0; j < 4; j++)
		{
			temp[j] = 0;
		}
		HPRAM_Write_Data_Buff(temp,i);
	}

	printf("\r\nFinished\r\n");

	return 0;
}

//Initializes UART0
void UartInit(void)
{
  UART_InitTypeDef UART_InitStruct;

  UART_InitStruct.UART_Mode.UARTMode_Tx = ENABLE;
  UART_InitStruct.UART_Mode.UARTMode_Rx = ENABLE;
  UART_InitStruct.UART_Int.UARTInt_Tx = DISABLE;
  UART_InitStruct.UART_Int.UARTInt_Rx = ENABLE;
  UART_InitStruct.UART_Ovr.UARTOvr_Tx = DISABLE;
  UART_InitStruct.UART_Ovr.UARTOvr_Rx = DISABLE;
  UART_InitStruct.UART_Hstm = DISABLE;
  UART_InitStruct.UART_BaudRate = 115200;//Baud Rate
	
  UART_Init(UART0,&UART_InitStruct);
}

/**
  * @brief  This function handles UART0 interrupt request.
  * @param  none
  * @retval none
  */
void UART0_Handler(void)
{
	char num = '0';

	if(UART_GetRxIRQStatus(UART0) == SET)
	{
		num  = UART_ReceiveChar(UART0);
		RXReceivedFlag = 1;

		if(Terminal)
		{
			if(num == 8) //Backspace
			{
				RXReceivedFlag = 2;
				RXBuf[RXpt]=0;
				RXpt--;
				if (RXpt <0)
				RXpt = 63;
			}
			else		//normal character
			{
				RXpt++;
				if(RXpt > 63)
					RXpt = 0;
				RXBuf[RXpt]= num;
			}
		}
		else
		{
			RXBuf[RXpt] = num;
		}
	}

	UART_ClearRxIRQ(UART0);
}
